@@ -711,7 +711,7 @@ g_main_context_init_pipe (GMainContext *context)
 }
 
 void
-_g_main_thread_init ()
+_g_main_thread_init (void)
 {
   GSList *curr = main_contexts_without_pipe;
   while (curr)
@@ -732,7 +732,7 @@ _g_main_thread_init ()
  * Return value: the new #GMainContext
  **/
 GMainContext *
-g_main_context_new ()
+g_main_context_new (void)
 {
   GMainContext *context = g_new0 (GMainContext, 1);
 
@@ -3716,6 +3716,8 @@ g_child_watch_source_new (GPid pid)
  * source is still active. Typically, you will want to call
  * g_spawn_close_pid() in the callback function for the source.
  * 
+ * GLib supports only a single callback per process id.
+ *
  * Return value: the id of event source.
  *
  * Since: 2.4
@@ -3758,6 +3760,8 @@ g_child_watch_add_full (gint            priority,
  * source is still active. Typically, you will want to call
  * g_spawn_close_pid() in the callback function for the source.
  *
+ * GLib supports only a single callback per process id.
+ *
  * Return value: the id of event source.
  *
  * Since: 2.4
