@@ -49,9 +49,6 @@
 
 #ifdef G_OS_WIN32
 #include <io.h>
-typedef FILE* GFileDescriptor;
-#else
-typedef gint GFileDescriptor;
 #endif
 
 /* --- structures --- */
@@ -90,20 +87,18 @@ static gpointer       default_log_data = NULL;
 #  define STRICT
 #  include <windows.h>
 #  undef STRICT
-#  include <process.h>          /* For _getpid() */
 static gboolean win32_keep_fatal_message = FALSE;
 
 /* This default message will usually be overwritten. */
-/* Yes, a fixed size buffer is bad. So sue me. But g_error is never
- * with huge strings, is it?
+/* Yes, a fixed size buffer is bad. So sue me. But g_error() is never
+ * called with huge strings, is it?
  */
 static gchar  fatal_msg_buf[1000] = "Unspecified fatal error encountered, aborting.";
 static gchar *fatal_msg_ptr = fatal_msg_buf;
 
-/* Just use stdio. If we're out of memory, we're hosed anyway. */
 #undef write
 static inline int
-dowrite (GFileDescriptor fd,
+dowrite (int          fd,
 	 const void  *buf,
 	 unsigned int len)
 {
@@ -115,65 +110,17 @@ dowrite (GFileDescriptor fd,
       return len;
     }
 
-  fwrite (buf, len, 1, fd);
-  fflush (fd);
+  write (fd, buf, len);
 
   return len;
 }
 #define write(fd, buf, len) dowrite(fd, buf, len)
 
-static void
-ensure_stdout_valid (void)
-{
-  static gboolean alloc_console_called = FALSE;
-  HANDLE handle;
-
-  if (win32_keep_fatal_message)
-    return;
-
-  if (!alloc_console_called)
-    {
-      handle = (HANDLE) _get_osfhandle (fileno (stdout)); 
-  
-      if (handle == INVALID_HANDLE_VALUE)
-	{
-	  AllocConsole ();
-	  alloc_console_called = TRUE;
-	  freopen ("CONOUT$", "w", stdout);
-	}
-    }
-}
-
-static void
-ensure_stderr_valid (void)
-{
-  static gboolean alloc_console_called = FALSE;
-  HANDLE handle;
-
-  if (win32_keep_fatal_message)
-    return;
-
-  if (!alloc_console_called)
-    {
-      handle = (HANDLE) _get_osfhandle (fileno (stderr)); 
-
-      if (handle == INVALID_HANDLE_VALUE)
-	{
-	  AllocConsole ();
-	  alloc_console_called = TRUE;
-	  freopen ("CONOUT$", "w", stderr);
-	}
-    }
-}
-
-#else
-#define ensure_stdout_valid()	/* Define as empty */
-#define ensure_stderr_valid()
 #endif
 
 static void
-write_string (GFileDescriptor fd,
-	      const gchar    *string)
+write_string (int          fd,
+	      const gchar *string)
 {
   write (fd, string, strlen (string));
 }
@@ -459,7 +406,7 @@ g_logv (const gchar   *log_domain,
 	  guint depth = GPOINTER_TO_UINT (g_private_get (g_log_depth));
 	  GLogDomain *domain;
 	  GLogFunc log_func;
-	  guint domain_fatal_mask;
+	  GLogLevelFlags domain_fatal_mask;
 	  gpointer data = NULL;
 
 	  if (was_fatal)
@@ -488,7 +435,7 @@ g_logv (const gchar   *log_domain,
 	  /* had to defer debug initialization until we can keep track of recursion */
 	  if (!(test_level & G_LOG_FLAG_RECURSION) && !_g_debug_initialized)
 	    {
-	      guint orig_test_level = test_level;
+	      GLogLevelFlags orig_test_level = test_level;
 
 	      _g_debug_init ();
 	      if ((domain_fatal_mask | g_log_always_fatal) & test_level)
@@ -579,6 +526,30 @@ g_return_if_fail_warning (const char *log_domain,
 	 expression);
 }
 
+void
+g_assert_warning (const char *log_domain,
+		  const char *file,
+		  const int   line,
+		  const char *pretty_function,
+		  const char *expression)
+{
+  /*
+   * Omit the prefix used by the PLT-reduction
+   * technique used in GTK+. 
+   */
+  if (g_str_has_prefix (pretty_function, "IA__"))
+    pretty_function += 4;
+  g_log (log_domain,
+	 G_LOG_LEVEL_ERROR,
+	 expression 
+	 ? "file %s: line %d (%s): assertion failed: (%s)"
+	 : "file %s: line %d (%s): should not be reached",
+	 file, 
+	 line, 
+	 pretty_function,
+	 expression);
+}
+
 #define CHAR_IS_SAFE(wc) (!((wc < 0x20 && wc != '\t' && wc != '\n' && wc != '\r') || \
 			    (wc == 0x7f) || \
 			    (wc >= 0x80 && wc < 0xa0)))
@@ -619,7 +590,6 @@ strdup_convert (const gchar *string,
 	  if (!warned)
 	    {
 	      warned = TRUE;
-	      ensure_stderr_valid ();
 	      _g_fprintf (stderr, "GLib: Cannot convert message: %s\n", err->message);
 	    }
 	  g_error_free (err);
@@ -705,9 +675,9 @@ format_unsigned (gchar  *buf,
 
 #define	ALERT_LEVELS		(G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING)
 
-static GFileDescriptor
-mklevel_prefix (gchar level_prefix[STRING_BUFFER_SIZE],
-		guint log_level)
+static int
+mklevel_prefix (gchar          level_prefix[STRING_BUFFER_SIZE],
+		GLogLevelFlags log_level)
 {
   gboolean to_stdout = TRUE;
 
@@ -754,20 +724,8 @@ mklevel_prefix (gchar level_prefix[STRING_BUFFER_SIZE],
 
 #ifdef G_OS_WIN32
   win32_keep_fatal_message = (log_level & G_LOG_FLAG_FATAL) != 0;
-
-  if (to_stdout)
-    {
-      ensure_stdout_valid ();
-      return stdout;
-    }
-  else
-    {
-      ensure_stderr_valid ();
-      return stderr;
-    }
-#else
-  return to_stdout ? 1 : 2;
 #endif
+  return to_stdout ? 1 : 2;
 }
 
 void
@@ -776,9 +734,12 @@ _g_log_fallback_handler (const gchar   *log_domain,
 			 const gchar   *message,
 			 gpointer       unused_data)
 {
-  gchar level_prefix[STRING_BUFFER_SIZE], pid_string[FORMAT_UNSIGNED_BUFSIZE];
+  gchar level_prefix[STRING_BUFFER_SIZE];
+#ifndef G_OS_WIN32
+  gchar pid_string[FORMAT_UNSIGNED_BUFSIZE];
+#endif
   gboolean is_fatal = (log_level & G_LOG_FLAG_FATAL) != 0;
-  GFileDescriptor fd;
+  int fd;
 
   /* we can not call _any_ GLib functions in this fallback handler,
    * which is why we skip UTF-8 conversion, etc.
@@ -791,15 +752,21 @@ _g_log_fallback_handler (const gchar   *log_domain,
   if (!message)
     message = "(NULL) message";
 
+#ifndef G_OS_WIN32
   format_unsigned (pid_string, getpid (), 10);
+#endif
 
   if (log_domain)
     write_string (fd, "\n");
   else
     write_string (fd, "\n** ");
+
+#ifndef G_OS_WIN32
   write_string (fd, "(process:");
   write_string (fd, pid_string);
   write_string (fd, "): ");
+#endif
+
   if (log_domain)
     {
       write_string (fd, log_domain);
@@ -883,7 +850,7 @@ g_log_default_handler (const gchar   *log_domain,
   gboolean is_fatal = (log_level & G_LOG_FLAG_FATAL) != 0;
   gchar level_prefix[STRING_BUFFER_SIZE], *string;
   GString *gstring;
-  GFileDescriptor fd;
+  int fd;
 
   /* we can be called externally with recursion for whatever reason */
   if (log_level & G_LOG_FLAG_RECURSION)
@@ -906,10 +873,53 @@ g_log_default_handler (const gchar   *log_domain,
     {
       const gchar *prg_name = g_get_prgname ();
       
+#ifdef G_OS_WIN32
+      if (prg_name)
+	prg_name = g_strdup (prg_name);
+      else
+	{
+	  if (G_WIN32_HAVE_WIDECHAR_API ())
+	    {
+	      wchar_t buf[MAX_PATH+1];
+	      if (GetModuleFileNameW (GetModuleHandle (NULL),
+				      buf, G_N_ELEMENTS (buf)) > 0)
+		{
+		  gchar *utf8_buf = g_utf16_to_utf8 (buf, -1,
+						     NULL, NULL, NULL);
+		  if (utf8_buf)
+		    {
+		      prg_name = g_path_get_basename (utf8_buf);
+		      g_free (utf8_buf);
+		    }
+		}
+	    }
+	  else
+	    {
+	      gchar buf[MAX_PATH+1];
+	      if (GetModuleFileNameA (GetModuleHandle (NULL),
+				      buf, G_N_ELEMENTS (buf)) > 0)
+		{
+		  gchar *locale_buf = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+		  if (locale_buf)
+		    {
+		      prg_name = g_path_get_basename (locale_buf);
+		      g_free (locale_buf);
+		    }
+		}
+	    }
+	}
+
+      if (prg_name)
+	{
+	  g_string_append_printf (gstring, "(%s): ", prg_name);
+	  g_free ((gchar *) prg_name);
+	}
+#else
       if (!prg_name)
 	g_string_append_printf (gstring, "(process:%lu): ", (gulong)getpid ());
       else
 	g_string_append_printf (gstring, "(%s:%lu): ", prg_name, (gulong)getpid ());
+#endif
     }
 
   if (log_domain)
@@ -989,7 +999,6 @@ g_print (const gchar *format,
     {
       const gchar *charset;
 
-      ensure_stdout_valid ();
       if (g_get_charset (&charset))
 	fputs (string, stdout); /* charset is UTF-8 already */
       else
@@ -1041,7 +1050,6 @@ g_printerr (const gchar *format,
     {
       const gchar *charset;
 
-      ensure_stderr_valid ();
       if (g_get_charset (&charset))
 	fputs (string, stderr); /* charset is UTF-8 already */
       else
