@@ -102,7 +102,7 @@ static void free_pending_nulls (GOptionContext *context,
 				gboolean        perform_nulls);
 
 GQuark
-g_option_context_error_quark (void)
+g_option_error_quark (void)
 {
   static GQuark q = 0;
   
@@ -428,18 +428,17 @@ get_change (GOptionContext *context,
       change = list->data;
 
       if (change->arg_data == arg_data)
-	break;
+	goto found;
     }
 
-  if (!change)
-    {
-      change = g_new0 (Change, 1);
-      change->arg_type = arg_type;
-      change->arg_data = arg_data;
-
-      context->changes = g_list_prepend (context->changes, change);
-    }
+  change = g_new0 (Change, 1);
+  change->arg_type = arg_type;
+  change->arg_data = arg_data;
   
+  context->changes = g_list_prepend (context->changes, change);
+  
+ found:
+
   return change;
 }
 
@@ -542,6 +541,7 @@ parse_arg (GOptionContext *context,
 	change->prev.str = *(gchar **)entry->arg_data;
 	change->allocated.str = data;
 
+	*(gchar **)entry->arg_data = data;
 	break;
       }
 
@@ -587,8 +587,8 @@ parse_arg (GOptionContext *context,
 			     entry->arg_data);
 	change->prev.integer = *(gint *)entry->arg_data;
 	*(gint *)entry->arg_data = data;
+	break;
       }
-      break;
     case G_OPTION_ARG_CALLBACK:
       {
 	gchar *tmp;
@@ -700,6 +700,7 @@ parse_long_option (GOptionContext *context,
 		     NULL, NULL, error);
 	  
 	  add_pending_null (context, &((*argv)[*index]), NULL);
+	  *parsed = TRUE;
 	}
       else
 	{
@@ -711,7 +712,7 @@ parse_long_option (GOptionContext *context,
 	      gchar *value = NULL;
 	      gchar *option_name;
 
-	      add_pending_null (context, &((*argv)[*index + 1]), NULL);
+	      add_pending_null (context, &((*argv)[*index]), NULL);
 	      
 	      if (arg[len] == '=')
 		value = arg + len + 1;
@@ -753,10 +754,6 @@ free_changes_list (GOptionContext *context,
 
       if (revert)
 	{
-	  /* Free any allocated data */
-	  g_free (change->allocated.str);
-	  g_strfreev (change->allocated.array.data);
-	  
 	  switch (change->arg_type)
 	    {
 	    case G_OPTION_ARG_NONE:
@@ -767,11 +764,14 @@ free_changes_list (GOptionContext *context,
 	      break;
 	    case G_OPTION_ARG_STRING:
 	    case G_OPTION_ARG_FILENAME:
+              g_free (change->allocated.str);
 	      *(gchar **)change->arg_data = change->prev.str;
 	      break;
 	    case G_OPTION_ARG_STRING_ARRAY:
 	    case G_OPTION_ARG_FILENAME_ARRAY:
+	      g_strfreev (change->allocated.array.data);
 	      *(gchar ***)change->arg_data = change->prev.array;
+	      break;
 	    default:
 	      g_assert_not_reached ();
 	    }
@@ -824,17 +824,13 @@ g_option_context_parse (GOptionContext   *context,
   GList *list;
 
   /* Set program name */
-  if (argc && argv)
+  if (argc && argv && *argc)
     {
       gchar *prgname;
       
-      prgname = strrchr ((*argv)[0], G_DIR_SEPARATOR);
-      if (prgname)
-	prgname++;
-      else
-	prgname = (*argv)[0];
-
+      prgname = g_path_get_basename ((*argv)[0]);
       g_set_prgname (prgname);
+      g_free (prgname);
     }
   else
     {
@@ -1083,8 +1079,8 @@ g_option_context_parse (GOptionContext   *context,
       GOptionGroup *group = list->data;
       
       if (group->error_func)
-	(* group->post_parse_func) (context, group,
-				    group->user_data, error);
+	(* group->error_func) (context, group,
+			       group->user_data, error);
       
       list = list->next;
     }
