@@ -43,6 +43,7 @@
 #include <signal.h>
 #endif
 
+#include "galias.h"
 #include "glib.h"
 #include "gprintf.h"
 #include "gprintfint.h"
@@ -1875,7 +1876,7 @@ g_strcasecmp (const gchar *s1,
 gint
 g_strncasecmp (const gchar *s1,
 	       const gchar *s2,
-	       gsize n)     
+	       guint n)     
 {
 #ifdef HAVE_STRNCASECMP
   return strncasecmp (s1, s2, n);
@@ -2207,11 +2208,11 @@ g_strsplit (const gchar *string,
  * contain any of the characters in @delimiters. If @max_tokens is reached, the
  * remainder is appended to the last token.
  *
- * For example the result of g_strtokenize ("abc:def/ghi", ":/", -1) is a
+ * For example the result of g_strsplit_set ("abc:def/ghi", ":/", -1) is a
  * %NULL-terminated vector containing the three strings "abc", "def", 
  * and "ghi".
  *
- * The result if g_strtokenize (":def/ghi:", ":/", -1) is a %NULL-terminated
+ * The result if g_strsplit_set (":def/ghi:", ":/", -1) is a %NULL-terminated
  * vector containing the four strings "", "def", "ghi", and "".
  * 
  * As a special case, the result of splitting the empty string "" is an empty
@@ -2219,7 +2220,7 @@ g_strsplit (const gchar *string,
  * special case is that being able to represent a empty vector is typically
  * more useful than consistent handling of empty elements. If you do need
  * to represent empty elements, you'll need to check for the empty string
- * before calling g_strsplit().
+ * before calling g_strsplit_set().
  *
  * Note that this function works on bytes not characters, so it can't be used 
  * to delimit UTF-8 strings for anything but ASCII characters.
@@ -2696,3 +2697,28 @@ g_strip_context  (const gchar *msgid,
   
   return msgval;
 }
+
+
+/**
+ * g_strv_length:
+ * @str_array: a %NULL-terminated array of strings.
+ * 
+ * Returns the length of the given %NULL-terminated 
+ * string array @str_array.
+ * 
+ * Return value: length of @str_array.
+ *
+ * Since: 2.6
+ **/
+guint
+g_strv_length (gchar **str_array)
+{
+  guint i = 0;
+
+  g_return_val_if_fail (str_array != NULL, 0);
+
+  while (str_array[i])
+    ++i;
+
+  return i;
+}
