@@ -30,8 +30,6 @@
 
 #include "config.h"
 
-#include <ctype.h>
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -72,7 +70,6 @@
 #  include <windows.h>
 #  undef STRICT
 #  include <lmcons.h>		/* For UNLEN */
-#  include <ctype.h>
 #endif /* G_PLATFORM_WIN32 */
 
 #ifdef G_OS_WIN32
@@ -88,13 +85,6 @@
 #include <libintl.h>
 #endif
 
-/* G_IS_DIR_SEPARATOR probably should be made public in GLib 2.4 */
-#ifdef G_OS_WIN32
-#define G_IS_DIR_SEPARATOR(c) (c == G_DIR_SEPARATOR || c == '/')
-#else
-#define G_IS_DIR_SEPARATOR(c) (c == G_DIR_SEPARATOR)
-#endif
-
 const guint glib_major_version = GLIB_MAJOR_VERSION;
 const guint glib_minor_version = GLIB_MINOR_VERSION;
 const guint glib_micro_version = GLIB_MICRO_VERSION;
@@ -436,7 +426,7 @@ g_parse_debug_string  (const gchar     *string,
 	    }
 	  
 	  for (i=0; i<nkeys; i++)
-	    if (g_ascii_strncasecmp(keys[i].key, p, q - p) == 0 &&
+	    if (g_ascii_strncasecmp (keys[i].key, p, q - p) == 0 &&
 		keys[i].key[q - p] == '\0')
 	      result |= keys[i].value;
 	  
@@ -712,12 +702,33 @@ g_get_current_dir (void)
       buffer[1] = 0;
     }
 
+#ifdef G_OS_WIN32
+  dir = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
+#else
   dir = g_strdup (buffer);
+#endif
   g_free (buffer);
   
   return dir;
 }
 
+#ifdef G_OS_WIN32
+
+#undef g_get_current_dir
+
+/* Binary compatibility version. Not for newly compiled code. */
+
+gchar*
+g_get_current_dir (void)
+{
+  gchar *utf8_dir = g_get_current_dir_utf8 ();
+  gchar *dir = g_locale_from_utf8 (utf8_dir, -1, NULL, NULL, NULL);
+  g_free (utf8_dir);
+  return dir;
+}
+
+#endif
+
 /**
  * g_getenv:
  * @variable: the environment variable to get.
@@ -1177,6 +1188,27 @@ g_get_home_dir (void)
   return g_home_dir;
 }
 
+#ifdef G_OS_WIN32
+
+#undef g_get_home_dir
+
+G_CONST_RETURN gchar*
+g_get_home_dir (void)
+{
+  static gchar *home_dir = NULL;
+
+  G_LOCK (g_utils_global);
+  if (!g_tmp_dir)
+    g_get_any_init ();
+  if (!home_dir && g_home_dir)
+    home_dir = g_locale_from_utf8 (g_home_dir, -1, NULL, NULL, NULL);
+  G_UNLOCK (g_utils_global);
+
+  return home_dir;
+}
+
+#endif
+
 /* Return a directory to be used to store temporary files. This is the
  * value of the TMPDIR, TMP or TEMP environment variables (they are
  * checked in that order). If none of those exist, use P_tmpdir from
@@ -1195,6 +1227,30 @@ g_get_tmp_dir (void)
   return g_tmp_dir;
 }
 
+#ifdef G_OS_WIN32
+
+#undef g_get_tmp_dir
+
+G_CONST_RETURN gchar*
+g_get_tmp_dir (void)
+{
+  static gchar *tmp_dir = NULL;
+
+  G_LOCK (g_utils_global);
+  if (!g_tmp_dir)
+    g_get_any_init ();
+  if (!tmp_dir)
+    tmp_dir = g_locale_from_utf8 (g_tmp_dir, -1, NULL, NULL, NULL);
+
+  if (tmp_dir == NULL)
+    tmp_dir = "C:\\";
+  G_UNLOCK (g_utils_global);
+
+  return tmp_dir;
+}
+
+#endif
+
 G_LOCK_DEFINE_STATIC (g_prgname);
 static gchar *g_prgname = NULL;
 
@@ -1667,8 +1723,8 @@ explode_locale (const gchar *locale,
  *       but it is big, ugly, and complicated, so I'm reluctant
  *       to do so when this should handle 99% of the time...
  */
-static GSList *
-compute_locale_variants (const gchar *locale)
+GSList *
+_g_compute_locale_variants (const gchar *locale)
 {
   GSList *retval = NULL;
 
@@ -1800,7 +1856,7 @@ g_get_language_names ()
       for (a = alist; *a; a++)
 	{
 	  gchar *b = unalias_lang (*a);
-	  list = g_slist_concat (list, compute_locale_variants (b));
+	  list = g_slist_concat (list, _g_compute_locale_variants (b));
 	}
       g_strfreev (alist);
       list = g_slist_append (list, "C");
