@@ -64,10 +64,10 @@ typedef int pid_t;
 #define pipe(phandles)	_pipe (phandles, 4096, _O_BINARY)
 
 /* For some POSIX functions that are not provided by the MS runtime,
- * we provide emulators in glib, which are prefixed with g_win32_.
+ * we provide emulation functions in glib, which are prefixed with
+ * g_win32_. Or that was the idea at some time, but there is just one
+ * of those:
  */
-#    define ftruncate(fd, size)	g_win32_ftruncate (fd, size)
-
 gint		g_win32_ftruncate	(gint		 f,
 					 guint		 size);
 #endif /* G_OS_WIN32 */
@@ -93,6 +93,11 @@ gchar*          g_win32_get_package_installation_subdirectory (gchar *package,
 							       gchar *dll_name,
 							       gchar *subdir);
 
+guint		g_win32_get_windows_version (void);
+
+#define G_WIN32_IS_NT_BASED() (g_win32_get_windows_version () < 0x80000000)
+#define G_WIN32_HAVE_WIDECHAR_API() (G_WIN32_IS_NT_BASED ())
+
 G_END_DECLS
 
 #endif	 /* G_PLATFORM_WIN32 */
